package rndclub.game.ganada;
import java.util.Date;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class Ganada extends Applet implements Runnable {
   public Frame baseFrame=null; 

   final static int LEVEL_MAX=10;

   final static int IMAGE_NUM=12;
   final static int IMAGE_NUM_ROW=8;
   final static int IMAGE_NUM_COL=10;
   final static int PANEL_NUM=IMAGE_NUM_ROW*IMAGE_NUM_COL;
   final static int TIMEOVER=80;
   final static int MISMATCH_DELAY=100;

   final static int DIRECTION_CENTER=0;
   final static int DIRECTION_NORTH=1;
   final static int DIRECTION_SOUTH=2;
   final static int DIRECTION_WEST=3;
   final static int DIRECTION_EAST=4;

   Panel gameBoard=null;
   ImageCanvas imageCanvas[]=null, oldImage=null;
   Image images[][]=null;

   Panel statusPanel=null;

   Button startStopButton=null;
   Label imageLabel=null;
   Choice imageChoice=null;
   Label elapsedTimeLabel=null;

   Label powerLabel=null;
   Label scoreLabel=null;
   TextField scoreField=null;

   public boolean gameFlag=false;
   public boolean itemStateChangedFlag = true;

   int matchedNum=0, selectedImage=0;
   long startTime=0, elapsedTime=0;
   Thread elapsedThread=null;
   int level=0, score=0;

   TimeBar timeBar=null;

   public void initBaseFrame() {
      int i=1;
      Component c = this;
      while(c!=null) {
         System.out.println("c-"+i+" : "+c.toString());
         if(c.toString().indexOf("frame")>=0) {
            baseFrame = (Frame)c;
         }
         c = c.getParent();
      }
      if(baseFrame==null) {
         baseFrame = new Frame();
      }
   }

   public void init() {
      initBaseFrame();

      gameBoard = new Panel(new GridLayout(IMAGE_NUM_ROW, IMAGE_NUM_COL, 2, 2));
      imageCanvas = new ImageCanvas[PANEL_NUM];

      int imageDirs = Integer.parseInt(getParameter("imageDirs"));
      images = new Image[imageDirs][IMAGE_NUM];

      WarningDialog dialog = new WarningDialog(baseFrame, "Notice!", "Please wait for loading images....");
      dialog.setVisible(true);

      MediaTracker tracker = new MediaTracker(this);
      String path=null;
      for(int i=0;i<imageDirs;i++) {
         for(int j=0;j<IMAGE_NUM;j++) {
            path = "images/"
                 + getParameter("imageDir["+i+"]")
                 + "/image" + (j+1) + ".jpg";
            images[i][j] = getImage(getDocumentBase(), path);

            System.out.println("getDocumentBase() : "+getDocumentBase().toString());
            System.out.println("path : "+path);

            tracker.addImage(images[i][j], 0);
/*
            System.out.println(images[i][j]);
*/
         }
      }
      try {
         tracker.waitForAll();
      } catch(InterruptedException e) {
         System.out.println(e);
      }

      setLayout(new BorderLayout(2, 2));
      for(int i=0;i<PANEL_NUM;i++) {
         imageCanvas[i] = new ImageCanvas(i);
         gameBoard.add(imageCanvas[i]);
      }

      statusPanel = new Panel();

      startStopButton = new Button("Start");
      imageLabel = new Label("Select:", Label.RIGHT);
      imageChoice = new Choice();
      elapsedTimeLabel = new Label("000", Label.LEFT);
      powerLabel = new Label("Power:", Label.RIGHT);
      timeBar = new TimeBar(TIMEOVER);
      scoreLabel = new Label("Score:", Label.RIGHT);
      scoreField = new TextField("0", 8);
      scoreField.setEnabled(false);

      imageLabel.setForeground(Color.blue);
      powerLabel.setForeground(Color.blue);
      scoreLabel.setForeground(Color.blue);
      elapsedTimeLabel.setForeground(Color.red);

      statusPanel.add(startStopButton);
      statusPanel.add(imageLabel);
      statusPanel.add(imageChoice);

      statusPanel.add(powerLabel);
      statusPanel.add(timeBar);
      statusPanel.add(elapsedTimeLabel);
      statusPanel.add(scoreLabel);
      statusPanel.add(scoreField);

      for(int i=0;i<imageDirs;i++) {
         imageChoice.addItem(getParameter("imageDir["+i+"]"));
      }

      add("Center", gameBoard);
      add("South", statusPanel);

      Button eastBar = new Button("");
      Button westBar = new Button("");
      Button northBar = new Button("");

      add("North", northBar);
      add("West", westBar);
      add("East", eastBar);

      northBar.setEnabled(false);
      westBar.setEnabled(false);
      eastBar.setEnabled(false);

      startStopButton.addMouseListener(new MouseAdapter() {
         public void mouseEntered(MouseEvent e) {
            e.getComponent().setForeground(Color.red);
         }
         public void mouseExited(MouseEvent e) {
            e.getComponent().setForeground(Color.black);
         }
      });

      startStopButton.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            clearGame();
            if(gameFlag == false) {
               startStopButton.setLabel("Stop");
               startGame();
               gameFlag = true;
            } else {
               startStopButton.setLabel("Start");
               stopGame();
               gameFlag = false;
            }
         }
      });

      imageChoice.addItemListener(new ItemListener() {
         public void itemStateChanged(ItemEvent e) {
            if(e.getStateChange() == ItemEvent.SELECTED) {
               selectedImage = imageChoice.getSelectedIndex();

               initGame();
            }
         }
      });

      MouseListener ml2 = new MouseAdapter() {
         public void mousePressed(MouseEvent e) {
            ImageCanvas currImage = (ImageCanvas)e.getSource();

            if(currImage != null) {
               currImage.select();
            }
            if((oldImage == null)||(oldImage == currImage)) {
               if(oldImage != null) {
                  oldImage.deselect();
               }
               oldImage = currImage;
            } else {
               if(oldImage.equals(currImage)) {
/*
                  System.out.println();
*/
                  boolean meetFlag = false;
                  meetFlag = meetRule(currImage.getImageCanvasId(),
                                      oldImage.getImageCanvasId(), DIRECTION_CENTER, 0);
                  if(meetFlag == true) {
                     // Chaeck they meet the rule of the game

                     oldImage.setState(ImageCanvas.STATE_BACK);
                     currImage.setState(ImageCanvas.STATE_BACK);

                     oldImage.repaint();
                     currImage.repaint();

                     oldImage.setEnabled(false);
                     currImage.setEnabled(false);

                     matchedNum += 2;
                     if(matchedNum >= IMAGE_NUM*4) {
                        congraturation();
                     }
                  } 
               } else {
                  // Mismatch sound
               }
               if(oldImage != null) {
                  oldImage.deselect();
               }
               if(currImage != null) {
                  currImage.deselect();
               }
               oldImage = null;
            }
         }
      };

      for(int i=0;i<PANEL_NUM;i++) {
         imageCanvas[i].addMouseListener(ml2);
         imageCanvas[i].setEnabled(false);
         imageCanvas[i].setImageCanvasId(i);
         imageCanvas[i].setState(ImageCanvas.STATE_BACK);
      }

      initGame();
   }

   public boolean meetRule(int i1, int i2, int direction, int turn) {
      int ox = i1 % IMAGE_NUM_COL, oy = i1 / IMAGE_NUM_COL;
      int mx = i2 % IMAGE_NUM_COL, my = i2 / IMAGE_NUM_COL;
      int distanceVector = Math.abs(mx - ox) + Math.abs(my - oy);

      if((i1 < 0)||(i1 >= PANEL_NUM)||
         (i2 < 0)||(i2 >= PANEL_NUM)||(turn > 2)) {
         return(false);
      }
/*
      imageCanvas[i2].select();
      try {
         Thread.sleep(200);
      } catch (InterruptedException e) { }
      imageCanvas[i2].deselect();

System.out.println("i1("+ox+","+oy+"), i2("+mx+","+my+"), dt("+direction+","+turn+")");
*/
      if((direction == DIRECTION_CENTER)&&(distanceVector == 1)) {
         return(true);
      } else if(direction != DIRECTION_CENTER) {
         if(i1 == i2) {
            return(true);
         } else if(imageCanvas[i2].getState() == ImageCanvas.STATE_FRONT) {
            return(false);
         }
      }

      boolean returnFlag=false;
      switch(direction) {
         case DIRECTION_CENTER:
// System.out.println();
            if(my > 0) {
               returnFlag = returnFlag || meetRule(i1, i2-IMAGE_NUM_COL, DIRECTION_NORTH, turn);
            }
            if(my < (IMAGE_NUM_ROW-1)) {
               returnFlag = returnFlag || meetRule(i1, i2+IMAGE_NUM_COL, DIRECTION_SOUTH, turn);
            }
            if(mx > 0) {
               returnFlag = returnFlag || meetRule(i1, i2-1, DIRECTION_WEST, turn);
            }
            if(mx < (IMAGE_NUM_COL-1)) {
               returnFlag = returnFlag || meetRule(i1, i2+1, DIRECTION_EAST, turn);
            }
         break;
         case DIRECTION_NORTH:
            if(my > 0) {
               returnFlag = returnFlag || meetRule(i1, i2-IMAGE_NUM_COL, DIRECTION_NORTH, turn);
            }
            if(mx > 0) {
               returnFlag = returnFlag || meetRule(i1, i2-1, DIRECTION_WEST, turn+1);
            }
            if(mx < (IMAGE_NUM_COL-1)) {
               returnFlag = returnFlag || meetRule(i1, i2+1, DIRECTION_EAST, turn+1);
            }
         break;
         case DIRECTION_SOUTH:
            if(my < (IMAGE_NUM_ROW-1)) {
               returnFlag = returnFlag || meetRule(i1, i2+IMAGE_NUM_COL, DIRECTION_SOUTH, turn);
            }
            if(mx > 0) {
               returnFlag = returnFlag || meetRule(i1, i2-1, DIRECTION_WEST, turn+1);
            }
            if(mx < (IMAGE_NUM_COL-1)) {
               returnFlag = returnFlag || meetRule(i1, i2+1, DIRECTION_EAST, turn+1);
            }
         break;
         case DIRECTION_EAST:
            if(my > 0) {
               returnFlag = returnFlag || meetRule(i1, i2-IMAGE_NUM_COL, DIRECTION_NORTH, turn+1);
            }
            if(my < (IMAGE_NUM_ROW-1)) {
               returnFlag = returnFlag || meetRule(i1, i2+IMAGE_NUM_COL, DIRECTION_SOUTH, turn+1);
            }
            if(mx < (IMAGE_NUM_COL-1)) {
               returnFlag = returnFlag || meetRule(i1, i2+1, DIRECTION_EAST, turn);
            }
         break;
         case DIRECTION_WEST:
            if(my > 0) {
               returnFlag = returnFlag || meetRule(i1, i2-IMAGE_NUM_COL, DIRECTION_NORTH, turn+1);
            }
            if(my < (IMAGE_NUM_ROW-1)) {
               returnFlag = returnFlag || meetRule(i1, i2+IMAGE_NUM_COL, DIRECTION_SOUTH, turn+1);
            }
            if(mx > 0) {
               returnFlag = returnFlag || meetRule(i1, i2-1, DIRECTION_WEST, turn);
            }
         break;
      }
      return(returnFlag);
   }

   public void congraturation() {
      stopGame();

      play(getDocumentBase(),"audio/gong.au");

      score += (100 * level + (TIMEOVER * 1000 - elapsedTime) / 100);
      scoreField.setText(""+score);
      level++;

      WarningDialog dialog=null;
      if(level > LEVEL_MAX) {
         dialog = new WarningDialog(baseFrame, "Notice!", "Congraturations!!");
         dialog.setVisible(true);
         dialog.setVisible(true);
         while(dialog.isVisible()) {
            try {
               Thread.sleep(100);
            } catch(InterruptedException e) {
            }
         }

         scoreBoard();
      } else {
         dialog = new WarningDialog(baseFrame, "Notice!", "Level "+level);
         dialog.setVisible(true);

         dialog.setVisible(true);
         while(dialog.isVisible()) {
            try {
               Thread.sleep(100);
            } catch(InterruptedException e) {
            }
         }

         startGame();
      }
   }

   public void scoreBoard() {
      AppletContext appletContext = getAppletContext();
      String queryString = getDocumentBase()+"ScoreBoard/ScoreBoard.asp?score="+score;

//      System.out.println(queryString);

      try {
         appletContext.showDocument(new URL(queryString), "_self");
      } catch(MalformedURLException ee) {
         appletContext.showStatus("URL Exception: " + ee);
      }
   }

   public void timeover() {
      stopGame();

      WarningDialog dialog = new WarningDialog(baseFrame, "Notice!", "Game Over");
      dialog.show();

            dialog.setVisible(true);
      while(dialog.isVisible()) {
         try {
            Thread.sleep(100);
         } catch(InterruptedException e) {
         }
      }

      score += (matchedNum * level * 10);
      scoreField.setText(""+score);
      scoreBoard();
   }

   public void clearGame() {
      score = 0;
      level = 0;
   }

   public void initGame() {
      int x=0, y=0, value=0;
      boolean check[] = new boolean[PANEL_NUM];

      for(int i=0;i<PANEL_NUM;i++) {
         check[i] = false;
         imageCanvas[i].setState(ImageCanvas.STATE_BACK);
      }

      for(int i=0;i<IMAGE_NUM*4;) {
         value = (int)(Math.random() * PANEL_NUM * 2) % PANEL_NUM;
         x = value % IMAGE_NUM_COL;
         y = value / IMAGE_NUM_COL;
         
         if((x > 0)&&(x < (IMAGE_NUM_COL-1))&&
            (y > 0)&&(y < (IMAGE_NUM_ROW-1))&&
            check[value] == false) {
            check[value] = true;
            imageCanvas[value].setImage(images[selectedImage][i%IMAGE_NUM]);
            imageCanvas[value].setId(i%IMAGE_NUM);
            imageCanvas[value].setState(ImageCanvas.STATE_FRONT);

            imageCanvas[value].repaint();

            i++;
         }
      }
      oldImage = null;
      matchedNum = 0;
   }

   public void startGame() {
      if(gameFlag == false) {
         initGame();

         gameFlag = true;
         imageChoice.setEnabled(false);
         for(int i=0;i<PANEL_NUM;i++) {
            if(imageCanvas[i].getState() == ImageCanvas.STATE_FRONT) {
               imageCanvas[i].setEnabled(true);
            }
         }
         timeBar.setTime(TIMEOVER - (5*level));

         startTime = new Date().getTime();
         start();
      }
      startStopButton.setLabel("Stop");
   }

   public void stopGame() {
      if(gameFlag == true) {
         stop();

         gameFlag = false;
         imageChoice.setEnabled(true);
         for(int i=0;i<PANEL_NUM;i++) {
            imageCanvas[i].setEnabled(false);
            imageCanvas[i].deselect();
         }
      }
      elapsedTimeLabel.setText("000");
      timeBar.setTime(0);
      startStopButton.setLabel("Start");
   }

   public void start() {
      if(gameFlag == true) {
         if(elapsedThread == null) {
            elapsedThread = new Thread(this);
         }
         elapsedThread.start();
      }
   }

   public void stop() {
      if(elapsedThread != null) {
         elapsedThread = null;
      }
   }

   public void run() {
      int time;

      while(true) {
         if(gameFlag == true) {
            elapsedTime = new Date().getTime() - startTime;
            time = (int)(TIMEOVER - (5 * level) - (int)(elapsedTime/1000));
   
            if(elapsedTimeLabel != null) {
               elapsedTimeLabel.setText(""+time);
            } else {
               elapsedTimeLabel.setText("000");
            }

            timeBar.setTime(time);
            if(time <= 0) {
               gameFlag = false;
               timeover();
               break;
            }
         }
         try {
            Thread.sleep(1000);
         } catch (InterruptedException e) { }
      }
//      elapsedThread = null;
   }
}
