/*
 * Project: MW Dilemma Game
 *    File: ServerPlayer.java
 *    Date: 2 10, 2001
 * Authhor: Yongwoo's Park.
 */

package rndclub.game.mw;
import java.io.*;
import java.net.*;
import java.util.*;

class ServerPlayer {
   public Socket socket=null;
   public BufferedReader is=null;
   public PrintWriter os=null;

   Lobby lobby=null;
   Group group=null;
   Room room=null;

   static final int ROOM_NUM=6;

   String groupId=null;
   String playerId=null;

   ServerPlayer(Socket socket, Lobby lobby, Room room) throws IOException {
      is = new BufferedReader(new InputStreamReader(socket.getInputStream()));
      os = new PrintWriter(socket.getOutputStream(), true);

      if(is == null) {
         throw new IOException();
      }
      if(os == null) {
         throw new IOException();
      }

      this.socket = socket;
      this.lobby = lobby;
      this.group = null;
      this.room = room;

      getInitialInfo();

      socket.setSoTimeout(10);
   }

   public void getInitialInfo() {
      if(is != null) {
         try {
            sendMessage("UserName");
            playerId = receiveMessage();
         } catch(SocketException e) {
            clear();
         } catch(IOException e) {
            clear();
         }
      }
   }

   public void sendMessage(String message) {
      if(message != null) {
         os.println(message);
         os.flush();
         if(os.checkError()) {
            lobby.removePlayer(this);
         }
         System.out.println("Send: "+message);
      }
   }

   public String receiveMessage() throws SocketException, IOException {
      String message=null;
      try {
         message = is.readLine();
         if(message == null) {
            throw(new IOException("Null pointer received...."));
         }
         System.out.println("Recv: "+message);
      } catch(SocketException e) {
         throw(e);
      } catch(InterruptedIOException e) {
         message = "";
      } catch(IOException e) {
         throw(e);
      } catch(Exception e) {
         message = "";
      }
      return(message);
   }

   protected void clear() {
      try {
         if(is != null) {
            is.close();
            is = null;
         }
      } catch(Exception e) {
         System.out.println(e);
         e.printStackTrace();
      }

      try {
         if(os != null) {
            os.close();
            os = null;
         }
      } catch(Exception e) {
         System.out.println(e);
      }

      try {
         if (socket != null) {
            socket.close();
            socket = null;
         }
      } catch (Exception e) {
         System.out.println(e);
         e.printStackTrace();
      }
   }

   public String toString() {
      return("Player["+playerId+"]: Socket["+socket+"], Room["+room+"]");
  }
}
