/*
 * Project: MW Dilemma Game
 *    File: Lobby.java
 *    Date: 2 10, 2001
 * Authhor: Yongwoo's Park.
 */

import java.io.*;
import java.net.*;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.StringTokenizer;

class Group {
   ServerPlayer owner=null;

   String groupId=null;
   String groupPass=null;
   String putMW=null;

   Vector playerList=null;

   int score=0;

   public Group(ServerPlayer player, String groupId, String groupPass) {
      this.playerList = new Vector();
      this.playerList.add(player);

      this.owner = player;

      this.groupId = groupId;
      this.groupPass = groupPass;

      this.putMW = "";
      this.score = 0;

      player.group = this;
      player.groupId = groupId;
   }

   public void add(ServerPlayer player) {
      player.group = this;
      player.groupId = groupId;
      playerList.add(player);
   }

   public void remove(ServerPlayer player) {
      playerList.remove(player);
      if((owner == player)&&(playerList.size() > 0)) {
         owner = (ServerPlayer)playerList.firstElement();
      }
      player.group = null;
      player.groupId = null;
   }

   public boolean isEmpty() {
      return(playerList.size() == 0);
   }
}

public class Lobby extends Thread {
   final static int ROOM_NUMBER = 6;

   Hashtable chatList=null;   // chatting list
   Hashtable roomList=null;   // room list
   Hashtable groupList=null;  // group (=team) list

   public Lobby() {
      chatList = new Hashtable();
      roomList = new Hashtable();
      groupList = new Hashtable();

      makeDeamonRoom("Room#1");
      makeDeamonRoom("Room#2");
      makeDeamonRoom("Room#3");
      makeDeamonRoom("Room#4");
      makeDeamonRoom("Room#5");
      makeDeamonRoom("Room#6");
   }

   public void makeDeamonRoom(String roomId) {
      Room room = new Room(roomId);
      roomList.put(roomId, room);

      room.start();
      System.out.println("Room daemon thread, "+room.roomId+" is started...");
   }

   public void addPlayer(ServerPlayer player) {
      if(player.playerId != null && chatList.get(player.playerId) == null){
         int i=0;
         String message=null;

         if(chatList.size() == 0) {
            System.out.println("Lobby thread is started...");
//            start();
         }
         chatList.put(player.playerId, player);

         message = "300|UNICAST|"+player.playerId+"|"+player.playerId+"|["+player.playerId+"]  ʽÿ.";
         player.sendMessage(message);

         message = "300|BROADCAST|"+player.playerId+"|*|["+player.playerId+"] Ͽϴ.";
         broadcastMessage(message);

         sendRoomList(player.playerId);
         broadcastUserList("");
      }
   }

   public void takePlayerFromRoom(ServerPlayer player, String roomId) {
      chatList.put(player.playerId, player);
      broadcastUserList("");

/*
      if(chatList.contains(player)) {
         String message = "300|BROADCAST|"+player.playerId+"|*|["+player.playerId+"] "+roomId+" ƿԽϴ..";
         broadcastMessage(message);
      }
*/
   }

   public void takePlayerIntoRoom(ServerPlayer player, String roomId) {
      chatList.remove(player.playerId);
      broadcastUserList("");

      String message = "300|BROADCAST|"+player.playerId+"|*|["+player.playerId+"] "+roomId+" Ͽϴ..";
      broadcastMessage(message);
   }

   public void removePlayer(ServerPlayer player) {
      chatList.remove(player.playerId);
      broadcastUserList("");

      String message = "300|BROADCAST|"+player.playerId+"|*|["+player.playerId+"] Ͽϴ..";
      broadcastMessage(message);
   }

   public ServerPlayer getPlayer(String playerId) {
      ServerPlayer player=null;

      if((player = (ServerPlayer)chatList.get(playerId)) != null) {
         return(player);
      }
      Enumeration enum = roomList.elements();
      while(enum.hasMoreElements()) {
         Room room = (Room)enum.nextElement();
         if((player = room.getPlayer(playerId)) != null) {
            return(player);
         }
      }
      return(null);
   }

   synchronized public Group getGroup(String groupId) {
      if(groupId == null) {
         return(null);
      }
      Enumeration enum = groupList.elements();
      while(enum.hasMoreElements()) {
         Group group = (Group)enum.nextElement();
         if(group.groupId.equals(groupId)) {
            return(group);
         }
      }
      return(null);
   }

   public void sendMessage(String playerId, String message) {
      ServerPlayer player = (ServerPlayer)chatList.get(playerId);
      if(player != null) {
         player.sendMessage(message);
      }
   }

   public void groupcastMessage(String groupId, String message) {
      Group group = getGroup(groupId);
      if((group == null)||(message == null)) {
         return;
      }

      Enumeration enum = group.playerList.elements();
      while(enum.hasMoreElements()) {
         ServerPlayer player = (ServerPlayer)enum.nextElement();
         player.sendMessage(message);
      }
   }

   public void broadcastMessage(String message) {
      if(message == null) {
         return;
      }
      ServerPlayer player=null;
      Enumeration enum = chatList.elements();
      while(enum.hasMoreElements()) {
         player = (ServerPlayer)enum.nextElement();
         if(player != null) {
            player.sendMessage(message);
         }
      }
   }

   public void broadcastRoomList() {
      Enumeration enum = roomList.elements();

      String message = "310|**SERVER**|,";
      while(enum.hasMoreElements()) {
         Room room = (Room)enum.nextElement();
         message += room.roomId + ",";
      }
      if(message.endsWith(",")) {
         message = message.substring(0, message.length()-1);
      }
      broadcastMessage(message);
   }

   public void sendRoomList(String playerId) {
      Enumeration enum = roomList.elements();

      String message = "310|"+playerId+"|,";
      while(enum.hasMoreElements()) {
         Room room = (Room)enum.nextElement();
         message += room.roomId + ",";
      }
      if(message.endsWith(",")) {
         message = message.substring(0, message.length()-1);
      }
      sendMessage(playerId, message);
   }

   public void sendUserList(String playerId, String roomId) {
      Enumeration enum=null;
      if("".equals(roomId)) {
         enum = chatList.elements();
      } else {
         Room room = (Room)roomList.get(roomId);
         if(room == null) {
            return;
         }
         enum = room.playerList.elements();
      }
      String message = "312|"+playerId+"|"+roomId+"|";
      while(enum.hasMoreElements()) {
         ServerPlayer player = (ServerPlayer)enum.nextElement();
         if(player.group != null) {
            message += player.playerId + " (" + player.groupId + ":" + player.group.score + "),";
         } else {
            message += player.playerId + " (׷:),";
         }
      }
      if(message.endsWith(",")) {
         message = message.substring(0, message.length()-1);
      }
      sendMessage(playerId, message);
   }

   public void broadcastUserList(String roomId) {
      Enumeration enum=null;
      if("".equals(roomId)) {
         enum = chatList.elements();
      } else {
         Room room = (Room)roomList.get(roomId);
         if(room == null) {
            return;
         }
         enum = room.playerList.elements();
      }
      String message = "312|**SERVER**|"+roomId+"|";
      while(enum.hasMoreElements()) {
         ServerPlayer player = (ServerPlayer)enum.nextElement();
         if(player.group != null) {
            message += player.playerId + " (" + player.groupId + ":" + player.group.score + "),";
         } else {
            message += player.playerId + " (׷:),";
         }
      }
      if(message.endsWith(",")) {
         message = message.substring(0, message.length()-1);
      }
      broadcastMessage(message);
   }

   public void makeRoom(String playerId, String roomId) {
      ServerPlayer player = (ServerPlayer)chatList.get(playerId);

      Room room = new Room(roomId, player);
      roomList.put(roomId, room);

      room.start();
      System.out.println("Room thread, "+room.roomId+" is started...");
   }

   public boolean enterRoom(String playerId, String roomId) {
      ServerPlayer player = (ServerPlayer)chatList.get(playerId);

      Room room = (Room)roomList.get(roomId);
      String message=null;
      if(room != null) {
         if((room.groupList.size() < 2)||(groupList.contains(player.group))) {
            message = "322|"+playerId+"|"+roomId+"|SUCCESS";
            player.sendMessage(message);

            takePlayerIntoRoom(player, roomId);
            room.addPlayer(player);
            broadcastUserList(room.roomId);

            return(true);
         } else {
            message = "322|"+playerId+"|"+roomId+"|FULL";
            player.sendMessage(message);

            // 뿡    ٴ  ޽
            message = "399|"+playerId+"|["+roomId+"]   ϴ.";
            player.sendMessage(message);
         }
      } else {
         message = "322|"+playerId+"|"+roomId+"|NOROOM";

         // ش   ʴ´ٴ  ޽
         message = "399|"+playerId+"|["+roomId+"]  ʽϴ.";
         sendMessage(playerId, message);
      }
      return(false);
   }

   public void removeRoom(String roomId) {
      Room room = (Room)roomList.get(roomId);
      if(room != null) {
         roomList.remove(roomId);
         room = null;
         broadcastRoomList();
      }
   }

   public void broadcastGroupList() {
      Enumeration enum = groupList.elements();

      String message = "314|**SERVER**|";
      while(enum.hasMoreElements()) {
         Group group = (Group)enum.nextElement();
         message += group.groupId + "(" + group.score + "),";
      }
      if(message.endsWith(",")) {
         message = message.substring(0, message.length()-1);
      }
      broadcastMessage(message);
   }

   public void sendGroupList(String playerId) {
      Enumeration enum = groupList.elements();

      String message = "314|**SERVER**|";
      while(enum.hasMoreElements()) {
         Group group = (Group)enum.nextElement();
         message += group.groupId + "(" + group.score + "),";
      }
      if(message.endsWith(",")) {
         message = message.substring(0, message.length()-1);
      }
      sendMessage(playerId, message);
   }

   public void broadcastGroupUserList(String groupId) {
      Enumeration enum=null;

      Group group = (Group)groupList.get(groupId);
      if(group == null) {
         return;
      }
      enum = group.playerList.elements();
      String message = "312|**SERVER**|"+groupId+"|";
      while(enum.hasMoreElements()) {
         ServerPlayer player = (ServerPlayer)enum.nextElement();
         if(player.group != null) {
            message += player.playerId + " (" + player.groupId + ":" + player.group.score + "),";
         } else {
            message += player.playerId + " (׷:),";
         }
      }
      if(message.endsWith(",")) {
         message = message.substring(0, message.length()-1);
      }
      broadcastMessage(message);
   }

   public void sendGroupUserList(String playerId, String groupId) {
      Enumeration enum=null;

      Group group = (Group)groupList.get(groupId);
      if(group == null) {
         return;
      }
      enum = group.playerList.elements();
      String message = "312|"+playerId+"|"+groupId+"|";
      while(enum.hasMoreElements()) {
         ServerPlayer player = getPlayer((String)enum.nextElement());
         if(player.group != null) {
            message += player.playerId + " (" + player.groupId + ":" + player.group.score + "),";
         } else {
            message += player.playerId + " (׷:),";
         }
      }
      if(message.endsWith(",")) {
         message = message.substring(0, message.length()-1);
      }
      sendMessage(playerId, message);
   }

   public void makeGroup(ServerPlayer player, String groupId, String groupPass) {
      if(groupList.containsKey(groupId)) {
         Group group = (Group)groupList.get(groupId);

         if(group.groupPass.equals(groupPass)) {
            group.add(player);
            sendMessage(player.playerId, "326|"+player.playerId+"|"+groupId+"|SUCCESS"); // ׷ ϱ  ޽ ȯ
            sendMessage(player.playerId, "399|"+player.playerId+"| ["+groupId+"] ׷쿡 մϴ.");

            broadcastUserList("");
         } else {
            // ޽ 
            sendMessage(player.playerId, "324|"+player.playerId+"|"+groupId+"|FAIL"); // ׷  
            sendMessage(player.playerId, "399|"+player.playerId+"|["+groupId+"] ׷ ̹ մϴ.");
         }
      } else {
         Group group = new Group(player, groupId, groupPass);
         groupList.put(groupId, group);

         sendMessage(player.playerId, "324|"+player.playerId+"|"+groupId+"|SUCCESS"); // ׷  
         System.out.println("Group, "+group.groupId+" is created...");

         broadcastUserList("");
      }
   }

   public void enterGroup(ServerPlayer player, String groupId, String groupPass) {
      if(groupList.containsKey(groupId)) {
         Group group = (Group)groupList.get(groupId);

         if(group.groupPass.equals(groupPass)) {
            group.add(player);
            sendMessage(player.playerId, "326|"+player.playerId+"|"+groupId+"|SUCCESS"); // ׷ ϱ 
            sendMessage(player.playerId, "399|"+player.playerId+"| ["+groupId+"] ׷쿡 մϴ.");

            broadcastUserList("");
         } else {
            // ޽ 
            sendMessage(player.playerId, "326|"+player.playerId+"|"+groupId+"|FAIL"); // ׷ ϱ 
            sendMessage(player.playerId, "399|"+player.playerId+"|["+groupId+"] ׷ ȣ ٸϴ.");
         }
      } else {
         sendMessage(player.playerId, "326|"+player.playerId+"|"+groupId+"|FAIL"); // ׷ ϱ 
         sendMessage(player.playerId, "399|"+player.playerId+"|["+groupId+"] ׷ ̸ Ʋϴ.");
      }
   }

   public void leaveGroup(ServerPlayer player) {
      if((player.groupId != null)&&(groupList.containsKey(player.groupId))) {
         Group group = (Group)groupList.get(player.groupId);
         group.remove(player);
         if(group.isEmpty()) {
            groupList.remove(group.groupId);
            System.out.println("Group, "+group.groupId+" is removed...");
         }
         System.out.println("399|"+player.playerId+"|["+player.groupId+"|"+group.isEmpty()+"]");
      } else {
         sendMessage(player.playerId, "399|"+player.playerId+"|["+player.groupId+"] ׷ ̸  Ʋϴ.");
      }
   }

   public void joinGame(ServerPlayer player) {
   }

   public void leaveGame(ServerPlayer player) {
      leaveGroup(player);

      removePlayer(player);
   }

   public void run() {
      while(true) {
         // Read Data
         try {
            sleep(100);
         } catch(InterruptedException ie) {
         }
         Enumeration e = chatList.elements();
         while(e.hasMoreElements()) {
            ServerPlayer player = (ServerPlayer)e.nextElement();
            try {
               String receive = null;
               receive = player.receiveMessage();

               if(receive == null || "".equals(receive)) {
                  continue;
               }

               // MW Game Protocol
               StringTokenizer st = new StringTokenizer(receive, "|");
               String command = st.nextToken();
               if("100".equals(command)) {
                  // εĳ ä 100|BROADCAST|playerId|*|chatStr
                  String type = st.nextToken();
                  String playerId = st.nextToken();
                  String target = st.nextToken();
                  String chatStr = st.nextToken();

                  String message = "300|BROADCAST|"+playerId+"|*|["+playerId+"] "+chatStr;
                  broadcastMessage(message);
               } else if("102".equals(command)) {
                  // ׷ĳ ä   102|GROUPCAST|playerId|groupId|chatStr
                  String type = st.nextToken();
                  String playerId = st.nextToken();
                  String groupId = st.nextToken();
                  String chatStr = st.nextToken();

                  String message = "302|GROUPCAST|"+playerId+"|"+groupId+"|[GROUP ("+groupId+")] "+chatStr;
                  groupcastMessage(groupId, message);
               } else if("104".equals(command)) {
                  // ĳ ä   104|[UNICAST|PRIVATE]|playerId|toPlayerId|chatStr
                  String type = st.nextToken();
                  String playerId = st.nextToken();
                  String target = st.nextToken();
                  String chatStr = st.nextToken();

                  String message = "304|UNICAST|"+playerId+"|"+target+"|["+playerId+"] "+chatStr;
                  sendMessage(target, message);
               } else if("110".equals(command)) {
                  // 븮Ʈû      110|playerId
                  String playerId = st.nextToken();

                  sendRoomList(playerId);
               } else if("112".equals(command)) {
                  // ڸƮû  112|playerId|roomId
                  String playerId = st.nextToken();
                  String roomId = st.nextToken();

                  sendUserList(playerId, roomId);
               } else if("114".equals(command)) {
                  // ׷츮Ʈû      114|playerId
                  String playerId = st.nextToken();

                  sendGroupList(playerId);
               } else if("116".equals(command)) {
                  // ׷ڸƮû  116|playerId|groupId
                  String playerId = st.nextToken();
                  String groupId = st.nextToken();

                  sendGroupUserList(playerId, groupId);
               } else if("120".equals(command)) {
                  // 游          120|playerId|roomId
                  String playerId = st.nextToken();
                  String roomId = st.nextToken();

                  makeRoom(playerId, roomId);
                  broadcastRoomList();
               } else if("122".equals(command)) {
                  //         122|playerId|roomId
                  String playerId = st.nextToken();
                  String roomId = st.nextToken();

                  if(enterRoom(playerId, roomId) == true) {
                     broadcastUserList("");
                  }
               } else if("124".equals(command)) {
                  // ׷츸        124|playerId|groupId|groupPass
                  String playerId = st.nextToken();
                  String groupId = st.nextToken();
                  String groupPass = st.nextToken();

                  ServerPlayer pp = getPlayer(playerId);
                  if(pp != null) {
                     makeGroup(pp, groupId, groupPass);
                  }
               } else if("126".equals(command)) {
                  // ׷        126|playerId|groupId|groupPass
                  String playerId = st.nextToken();
                  String groupId = st.nextToken();
                  String groupPass = st.nextToken();

                  ServerPlayer pp = getPlayer(playerId);
                  if(pp != null) {
                     enterGroup(pp, groupId, groupPass);
                  }
               } else if("180".equals(command)) {
                  //           180|JOIN|playerId
                  String type = st.nextToken();
                  String playerId = st.nextToken();

                  ServerPlayer pp = getPlayer(playerId);
                  if(pp != null) {
                     joinGame(pp);
                  }
               } else if("182".equals(command)) {
                  //           182|LEAVE|playerId
                  String type = st.nextToken();
                  String playerId = st.nextToken();

                  ServerPlayer pp = getPlayer(playerId);
                  if(pp != null) {
                     leaveGame(pp);
                  }
               }
            } catch(SocketException ne) {
               leaveGroup(player);
               removePlayer(player);
            } catch(IOException ne) {
               leaveGroup(player);
               removePlayer(player);
            }
         }
      }
//      System.out.println("Lobby thread is finished...");
   }

   public void destroyPlayer(ServerPlayer player) {
      try {
         if(player.is != null) {
            player.is.close();
            player.is = null;
         }
      } catch (Exception e) {
      }
      try {
         if(player.os != null) {
            player.os.close();
            player.os = null;
         }
		 } catch (Exception e) {
      }
      try {
         if(player.socket != null) {
            player.socket.close();
            player.socket = null;
         }
      } catch(Exception e) {
      }
   }
}
