/*
 * Project: MW Dilemma Game
 *    File: Room.java
 *    Date: 2 10, 2001
 * Authhor: Yongwoo's Park.
 */

import java.net.*;
import java.io.*;
import java.util.*;

class Room extends Thread {
   static int roomCount=0;
   final static int GROUP_NUMBER = 2;
   final static int MATCH_NUMBER = 8;

   String roomId=null;
   ServerPlayer hostPlayer=null;

   Vector groupList=null;
   Vector playerList=null;
   int putCount=0;
   int matchCount=0;

   boolean inGame = false;

   public Room(String roomId) {
      this.roomId = roomId;
      this.hostPlayer = null;

      groupList = new Vector();
      playerList = new Vector();

      System.out.println("ROOM(): "+roomId);
   }

   public Room(String roomId, ServerPlayer player) {
      this.roomId = roomId;
      this.hostPlayer = player;

      groupList = new Vector();
      playerList = new Vector();

      addPlayer(player);

      System.out.println("ROOM(String, ServerPlayer): "+roomId);
   }

   synchronized public void addPlayer(ServerPlayer player) {
      if(!playerList.contains(player)) {
         playerList.addElement(player);
         player.room = this;

         broadcastUserList();

         String message = "799|**SERVER**|["+player.playerId+" ("+player.groupId+")] Ͽϴ..";
         broadcastMessage(message);

         message = "721|INIT|"+player.playerId;
         sendMessage(player.playerId, message);

         if(player.group != null) {
            if(!groupList.contains(player.group)) {
               groupList.addElement(player.group);
            }

            if(inGame == false) {
               Enumeration enum = playerList.elements();
               int ownerCount=0;
               while(enum.hasMoreElements()) {
                  ServerPlayer pp = (ServerPlayer)enum.nextElement();
                  if((pp.group != null)&&(pp.group.owner == pp)) {
                     if((!groupList.contains(pp.group))&&(groupList.size() < 2)) {
                        groupList.addElement(pp.group);
                     }
                     ownerCount++;
                     if(ownerCount >= 2) {
                        clearGame();
                        startGame();
                     }
                  }
               }
            }
         }
      }
   }

   synchronized public void removePlayer(ServerPlayer player) {
      if(player == null) {
         return;
      }
      player.room = null;

      if(playerList.contains(player)) {
         playerList.removeElement(player);
         player.lobby.takePlayerFromRoom(player, roomId);
/*
 *   .
         if(playerList.size() == 0) {
            player.lobby.removeRoom(roomId);
         }
*/

         broadcastUserList();

         String message = "799|**SERVER**|["+player.playerId+" ("+player.groupId+")] Ͽϴ..";
         broadcastMessage(message);

         if(player.group != null) {
            boolean groupPlayerFlag = true;
            Enumeration enum = playerList.elements();
            ServerPlayer pp = null;
            while(enum.hasMoreElements()) {
               pp = (ServerPlayer)enum.nextElement();
               if((pp.group != null)&&(pp.group == player.group)) {
                  groupPlayerFlag = false;
                  break;
               }
            }
            if(groupPlayerFlag == true) {
               groupList.removeElement(player.group);
               if(groupList.size() < 2) {
                  message = "799|**SERVER**|ٸ    ٸϴ..";
                  broadcastMessage(message);

                  clearGame();
               }
            } else {
               // ,   ÷̾ ׷  ÷̾  
               //  ÷̾     (pp) Ѵ.
               if((player.group.owner == player)&&(inGame == true)) {
                  player.group.owner = pp;

                  message = "799|**SERVER**|["+player.groupId+"]  ǥ ["+pp.playerId+"] ٲϴ..";
                  broadcastMessage(message);

                  startGame();
               }
            }
         }

         message = "782|EXIT_ROOM|"+player.playerId;
         player.sendMessage(message);

         message = "399|**SERVER**|["+player.playerId+" ("+player.groupId+")] ǵ Խϴ..";
         player.sendMessage(message);
      }
   }

   synchronized public void removePlayer(String playerId) {
      ServerPlayer player = getPlayer(playerId);
      removePlayer(player);
   }

   synchronized public void removeAllToLobby() {
      Enumeration enum = playerList.elements();
      while(enum.hasMoreElements()) {
         ServerPlayer player = (ServerPlayer)enum.nextElement();

         player.lobby.takePlayerFromRoom(player, roomId);
         player.room = null;

         String message = "782|EXIT_ROOM|"+player.playerId;
         player.sendMessage(message);

         message = "399|**SERVER**|["+player.playerId+" ("+player.groupId+")] ǵ Խϴ..";
         player.sendMessage(message);
      }

      playerList.removeAllElements();
      groupList.removeAllElements();
      clearGame();

      broadcastUserList();
   }

   synchronized public ServerPlayer getPlayer(String playerId) {
      if(playerId == null) {
         return(null);
      }
      Enumeration enum = playerList.elements();
      while(enum.hasMoreElements()) {
         ServerPlayer player = (ServerPlayer)enum.nextElement();
         if(player.playerId.equals(playerId)) {
            return(player);
         }
      }
      return(null);
   }

   synchronized public Group getGroup(String groupId) {
      if(groupId == null) {
         return(null);
      }
      Enumeration enum = groupList.elements();
      while(enum.hasMoreElements()) {
         Group group = (Group)enum.nextElement();
         if(group.groupId.equals(groupId)) {
            return(group);
         }
      }
      return(null);
   }

   public void sendMessage(String playerId, String message) {
      ServerPlayer player = getPlayer(playerId);
      if((player == null)||(message == null)) {
         return;
      }
      player.sendMessage(message);
   }

   public void groupcastMessage(String groupId, String message) {
      Group group = getGroup(groupId);
      if((group == null)||(message == null)) {
         return;
      }

      Enumeration enum = group.playerList.elements();
      while(enum.hasMoreElements()) {
         ServerPlayer player = (ServerPlayer)enum.nextElement();
         player.sendMessage(message);
      }
   }

   public void broadcastMessage(String message) {
      if(message == null) {
         return;
      }
      Enumeration enum = playerList.elements();
      while(enum.hasMoreElements()) {
         ServerPlayer player = (ServerPlayer)enum.nextElement();
         player.sendMessage(message);
      }
   }

   void sendUserList(String playerId) {
      ServerPlayer player = getPlayer(playerId);
      if(player != null) {
         sendUserList(player);
      }
   }

   void sendUserList(ServerPlayer player) {
      Enumeration enum = playerList.elements();
      String message = "**SERVER**|"+roomId+"|";
      while(enum.hasMoreElements()) {
         ServerPlayer pp = (ServerPlayer)enum.nextElement();
         if(pp.group != null) {
            message += pp.playerId + " (" + pp.groupId + ":" + pp.group.score + "),";
         } else {
            message += pp.playerId + " (׷:),";
         }
      }
      if(message.endsWith(",")) {
         message = message.substring(0, message.length()-1);
      }
      player.sendMessage("312|"+message);
      player.sendMessage("712|"+message);
   }

   void broadcastUserList() {
      Enumeration enum = playerList.elements();
      String message = "**SERVER**|"+roomId+"|";
      while(enum.hasMoreElements()) {
         ServerPlayer pp = (ServerPlayer)enum.nextElement();
         if(pp.group != null) {
            message += pp.playerId + " (" + pp.groupId + ":" + pp.group.score + "),";
         } else {
            message += pp.playerId + " (׷:),";
         }
      }
      if(message.endsWith(",")) {
         message = message.substring(0, message.length()-1);
      }
      broadcastMessage("312|"+message);
      broadcastMessage("712|"+message);
   }

   synchronized public void clearPae() {
      Enumeration enum = groupList.elements();
      while(enum.hasMoreElements()) {
         Group group = (Group)enum.nextElement();
         group.putMW = null;
      }
      putCount = 0;
   }

   synchronized public void clearGame() {
      inGame = false;
      matchCount = 0;
   }

   synchronized public void startGame() {
      String message=null;

      inGame = true;
      clearPae();

      message = "799|**SERVER**|M-W   ["+(matchCount+1)+"/"+MATCH_NUMBER+"]  ()մϴ.";
      broadcastMessage(message);

      message = "720|START|*";

      Group group0 = (Group)groupList.elementAt(0);
      Group group1 = (Group)groupList.elementAt(1);

      sendMessage(group0.owner.playerId, message);
      sendMessage(group1.owner.playerId, message);

      sendPoint();
   }

   synchronized public void putPae(String groupId, String mw) {
      String message=null;

      Group group0 = (Group)groupList.elementAt(0);
      Group group1 = (Group)groupList.elementAt(1);

      if(groupId.equals(group0.groupId)) {
         putCount++;
         group0.putMW = mw;
      } else if(groupId.equals(group1.groupId)) {
         putCount++;
         group1.putMW = mw;
      }

      if(putCount == 2) {
         finishGame();
         calculatePoint();
         sendPoint();

         broadcastUserList();
         group0.owner.lobby.broadcastGroupList();

         matchCount++;
         if(matchCount >= MATCH_NUMBER) {
            message = "799|**SERVER**|M-W   ϴ.";
            broadcastMessage(message);

            message = "799|**SERVER**|5 , [] ڵ ̵մϴ.";
            broadcastMessage(message);
            try {
               Thread.sleep(5000);  //  5  
            } catch(InterruptedException e) {
            }

            removeAllToLobby();
         } else {
            try {
               Thread.sleep(3000);  //  3  
            } catch(InterruptedException e) {
            }

            startGame();
         }
      }
      System.out.println();
   }

   synchronized public void finishGame() {
      String message=null;

      message = "799|**SERVER**|M-W   ["+(matchCount+1)+"/"+MATCH_NUMBER+"]  մϴ.";
      broadcastMessage(message);

      Group group0 = (Group)groupList.elementAt(0);
      Group group1 = (Group)groupList.elementAt(1);

      message = "722|PUT|"+group0.groupId+","+group0.putMW+"|"+group1.groupId+","+group1.putMW;
      broadcastMessage(message);
   }

   synchronized public void calculatePoint() {
      int score0=0, score1=0;
      String message=null;

      Group group0 = (Group)groupList.elementAt(0);
      Group group1 = (Group)groupList.elementAt(1);

      if(("W".equals(group0.putMW))&&("W".equals(group1.putMW))) {
         score0 = +3;
         score1 = +3;
      } else if(("W".equals(group0.putMW))&&("M".equals(group1.putMW))) {
         score0 = -6;
         score1 = +6;
      } else if(("M".equals(group0.putMW))&&("W".equals(group1.putMW))) {
         score0 = +6;
         score1 = -6;
      } else if(("M".equals(group0.putMW))&&("M".equals(group1.putMW))) {
         score0 = -3;
         score1 = -3;
      }
      group0.score += score0;
      group1.score += score1;
   }

   synchronized public void sendPoint() {
      String message=null;

      Group group0 = (Group)groupList.elementAt(0);
      Group group1 = (Group)groupList.elementAt(1);

      message = "724|POINT|"+group0.groupId+","+group0.score+"|"+group1.groupId+","+group1.score;

      broadcastMessage(message);
   }

   synchronized public void joinGame(ServerPlayer player) {
      String message = "799|**SERVER**|["+player.playerId+" ("+player.groupId+")] Ͽϴ..";
      broadcastMessage(message);

      broadcastUserList();
   }

   public void run() {
      String receive=null, message=null;
      Enumeration enum=null;
      ServerPlayer player=null;

      while(true) {
         // Read Data
         try {
            Thread.sleep(50);
         } catch(InterruptedException ie) {
         }
         enum = playerList.elements();
         while(enum.hasMoreElements()) {
            player = (ServerPlayer)enum.nextElement();
            try {
               receive = player.receiveMessage();
               if(receive == null || "".equals(receive)) {
                  continue;
               }
               // MW Game Protocol
               // nnn|arg1|arg2|arg3|...
               StringTokenizer st = new StringTokenizer(receive, "|");
               String command = st.nextToken();
               if("500".equals(command)) {
                  // εĳ ä 500|BROADCAST|playerId|*|chatStr
                  String type = st.nextToken();
                  String playerId = st.nextToken();
                  String target = st.nextToken();
                  String chatStr = st.nextToken();

                  message = "700|BROADCAST|"+playerId+"|*|["+playerId+"] "+chatStr;
                  broadcastMessage(message);
               } else if("502".equals(command)) {
                  // ׷ĳ ä   502|GROUPCAST|playerId|groupId|chatStr
                  String type = st.nextToken();
                  String playerId = st.nextToken();
                  String groupId = st.nextToken();
                  String chatStr = st.nextToken();

                  message = "702|GROUPCAST|"+playerId+"|"+groupId+"|[GROUP ("+groupId+")] "+chatStr;
                  groupcastMessage(groupId, message);
               } else if("504".equals(command)) {
                  // ĳ ä   504|[UNICAST|PRIVATE]|playerId|toPlayerId|chatStr
                  String type = st.nextToken();
                  String playerId = st.nextToken();
                  String toPlayerId = st.nextToken();
                  String chatStr = st.nextToken();

                  message = "704|UNICAST|"+playerId+"|"+toPlayerId+"|["+playerId+"] "+chatStr;
                  sendMessage(toPlayerId, message);
               } else if("512".equals(command)) {
                  // ڸƮ       512|playerId|roomId
                  String playerId = st.nextToken();
                  String roomId = st.nextToken();

                  player = getPlayer(playerId);
                  if(player != null) {
                     sendUserList(player);
                  }
               } else if("520".equals(command)) {
                  // ӽ       520|START|groupId
                  String type = st.nextToken();
                  String groupId = st.nextToken();

//                  startGame();
               } else if("522".equals(command)) {
                  // Ӱ      522|PUT|groupId|[M|W]
                  String type = st.nextToken();
                  String groupId = st.nextToken();
                  String mw = st.nextToken();

                  message = "723|PUT|"+groupId+"|"+mw;
                  groupcastMessage(groupId, message);

                  putPae(groupId, mw);
               } else if("524".equals(command)) {
                  //         524|POINT|groupId|point1
                  String type = st.nextToken();
                  String groupId = st.nextToken();
                  String point = st.nextToken();

                  sendPoint();

                  broadcastUserList();
               } else if("580".equals(command)) {
                  //            580|JOIN_ROOM|playerId
                  String type = st.nextToken();
                  String playerId = st.nextToken();

                  message = "780|JOIN_ROOM|"+playerId;
                  sendMessage(playerId, message);

                  broadcastUserList();
               } else if("582".equals(command)) {
                  //             582|EXIT_ROOM|playerId
                  String type = st.nextToken();
                  String playerId = st.nextToken();

                  removePlayer(playerId);
               }
            } catch(SocketException e) { //Stream doesn't exist.
               removePlayer(player);
            } catch(IOException e) { //Stream doesn't exist.
               removePlayer(player);
            } catch(Exception e) {
            }
         }
      }
   }
}
