import java.awt.*;
import java.awt.event.*;
import java.applet.*;

import java.io.*;
import java.net.*;
import java.util.Date;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

class TimeBar extends Canvas {
   int time, maxTime;
   Color powerColor=null, bgColor=null;

   public TimeBar(int maxTime) {
      this.maxTime = maxTime;
      time = 0;
      powerColor = new Color(0xff, 0x99, 0x00);
      bgColor = new Color(0x22, 0x22, 0x66);
      setSize(maxTime*3, 16);
   }

   int getTime() {
      return(time);
   }

   void setTime(int time) {
      this.time = time;
      repaint();
   }

   void setMaxTime(int maxTime) {
      this.maxTime = maxTime;
      repaint();
   }

   public void repaint() {
      update(getGraphics());
   }

   public void update(Graphics g) {
      paint(g);
   }

   public void paint(Graphics g) {
      if(g == null) return;

      for(int i=0;i<time;i++) {
         if(powerColor != null) {
            g.setColor(powerColor);
         } else {
            g.setColor(Color.red);
         }
         g.drawLine(i*3, 0, i*3, 15);;
         g.drawLine(i*3+1, 0, i*3+1, 15);;
         g.setColor(Color.lightGray);
         g.drawLine(i*3+2, 0, i*3+2, 15);;
      }
      for(int i=time;i<maxTime;i++) {
         if(bgColor != null) {
            g.setColor(bgColor);
         } else {
            g.setColor(Color.lightGray);
         }
         g.drawLine(i*3, 0, i*3, 15);;
         g.drawLine(i*3+1, 0, i*3+1, 15);;
         g.setColor(Color.black);
         g.drawLine(i*3+2, 0, i*3+2, 15);;
      }

      String str = ""+time;
      FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(getFont());
      if((getSize().width >> 1) > time*3) {
         g.setColor(Color.white);
      } else {
         g.setColor(Color.blue);
      }
      g.drawString(str, getSize().width - fm.stringWidth(str) >> 1, 12);
   }
}

class ImageCanvas extends Canvas {
   final static int STATE_FRONT=0;
   final static int STATE_BACK=1;
   final static int INVALID_ID=-1;

   Color bgColor=null;

   Image image=null;
   int id=INVALID_ID, width, height;
   int state=STATE_BACK;

   public ImageCanvas() {
      this.id = INVALID_ID;
      this.image = null;
      init();
   }

   public ImageCanvas(int id, Image image) {
      this.id = id;
      this.image = image;
      init();
   }

   public void init() {
      state = STATE_BACK;
      if(image == null) {
         setSize(0, 0);
      }
      bgColor = new Color(0xef, 0xf8, 0xff);
   }

   public boolean equals(Object o) {
      if((o != null)&&(o instanceof ImageCanvas)) {
         ImageCanvas p = (ImageCanvas)o;
         if(p.getId() == this.id) {
            return(true);
         }
      }
      return(false);
   }

   public int getState() {
      return(state);
   }

   public int getId() {
      return(id);
   }

   public void setId(int id) {
      this.id = id;
   }

   public void reverseState() {
      if(state == STATE_FRONT) {
         state = STATE_BACK;
      } else {
         state = STATE_FRONT;
      }
   }

   public void setState(int state) {
      this.state = state;
   }

   public Image getImage() {
      return(image);
   }

   public void setImage(Image image) {
      this.image = image;
      init();
   }

   public int getWidth() {
      return(getSize().width);
   }

   public int getHeight() {
      return(getSize().height);
   }

   public void repaint() {
      paint(getGraphics());
   }

   public void update(Graphics g) {
      paint(g);
   }

   public void paint(Graphics g) {
      int width = getSize().width;
      int height = getSize().height;

      if(g == null) return;

      if((image != null)&&(state == STATE_FRONT)) {
         g.drawImage(image, 0, 0, width, height, this);
      } else {
         if(bgColor != null) {
            g.setColor(bgColor);
         } else {
            g.setColor(Color.lightGray);
         }
         g.fillRect(0, 0, width-1, height-1);
         g.setColor(Color.black);
         g.drawRect(0, 0, width-1, height-1);
      }
   }
}

class WarningDialog extends Dialog {
   Button okButton=null, msgButton=null;

   public WarningDialog(Frame owner, String title, String message) {
      super(owner, title, true);

      setLayout(new BorderLayout());

      msgButton = new Button(message);
      okButton = new Button("OK");

      Panel p = new Panel();
      p.add(okButton);

      add("Center", msgButton);
      add("South", p);

      okButton.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            dispose();
         }
      });

      addWindowListener(new WindowAdapter() {
         public void windowClosing(WindowEvent e) {
            dispose();
         }
      });

      okButton.addMouseListener(new MouseAdapter() {
         public void mouseEntered(MouseEvent e) {
            e.getComponent().setForeground(Color.red);
         }
         public void mouseExited(MouseEvent e) {
            e.getComponent().setForeground(Color.black);
         }
      });

      pack();
      if(getSize().width < 300) {
         setSize(320, 120);
      }
      autoAlign();

      okButton.requestFocus();
   }

   public void autoAlign() {
      int w = Toolkit.getDefaultToolkit().getScreenSize().width;
      int h = Toolkit.getDefaultToolkit().getScreenSize().height;
      int x = (((w - getSize().width) / 2) < 0) ? 0 : ((w - getSize().width) / 2);
      int y = (((h - getSize().height) / 2) < 0) ? 0 : ((h - getSize().height) / 2);

      setLocation(x, y);
   }

   public void setVisible(boolean b) {
      if(b) {
         show();
         toFront();
         requestFocus();
      } else {
         hide();
      }
   }
}

class LoadingBarWindow extends Dialog {
   Label messageLabel=null;
   TimeBar loadingBar=null;
   ImageCanvas logo=null;

   public LoadingBarWindow(Frame owner, Image logoImage, String message) {
      super(owner, "ٿε º", false);

      setLayout(new BorderLayout(4, 4));

      setBackground(Color.black);

      logo = new ImageCanvas(0, logoImage);
      logo.setSize(logoImage.getWidth(logo), logoImage.getHeight(logo));
      logo.setState(ImageCanvas.STATE_FRONT);

      Panel logoPanel = new Panel();
      logoPanel.add(logo);

      messageLabel = new Label(message);
      messageLabel.setBackground(Color.lightGray);

      loadingBar = new TimeBar(100);
      Panel barPanel = new Panel();
      barPanel.add(loadingBar);

      Panel statusPanel = new Panel(new BorderLayout(4, 4));
      statusPanel.add("North", messageLabel);
      statusPanel.add("Center", barPanel);

      add("Center", logoPanel);
      add("South", statusPanel);

      pack();

      setSize(getSize().width+16, getSize().height+8);

      autoAlign();

      addWindowListener(new WindowAdapter() {
         public void windowClosing(WindowEvent e) {
            dispose();
         }
      });
   }

   public void setStatusLevel(int statusLevel) {
      loadingBar.setTime(statusLevel);
   }

   public void setMessage(String message) {
      messageLabel.setText(message);
   }

   public void setLogo(Image logoImage) {
      logo.setImage(logoImage);
   }

   public void autoAlign() {
      int w = Toolkit.getDefaultToolkit().getScreenSize().width;
      int h = Toolkit.getDefaultToolkit().getScreenSize().height;
      int x = (((w - getSize().width) / 2) < 0) ? 0 : ((w - getSize().width) / 2);
      int y = (((h - getSize().height) / 2) < 0) ? 0 : ((h - getSize().height) / 2);

      setLocation(x, y);
   }
}

public class SamePiece extends Applet implements Runnable {
   final static int LEVEL_MAX=10;

   final static int IMAGE_NUM=12;
   final static int IMAGE_NUM_ROW=4;
   final static int IMAGE_NUM_COL=6;
   final static int PANEL_NUM=IMAGE_NUM_ROW*IMAGE_NUM_COL;
   final static int TIMEOVER=60;
   final static int TIMEDEADLINE_DOWN=5;
   final static int MISMATCH_DELAY=250;

   Panel gameBoard=null;
   ImageCanvas imageCanvas[]=null, oldImage=null;
   Image images[][]=null;
   Button startStopButton=null;

   Panel statusPanel=null;

   Label imageLabel=null;
   Choice imageChoice=null;

   Label powerLabel=null;
   Label scoreLabel=null;
   TextField scoreField=null;

   public boolean gameFlag=false;

   int currentFrontNum=0, matchedNum=0, selectedImage=0;
   long startTime=0, elapsedTime=0;
   Thread elapsedThread=null;
   int level=0, score=0;
   int imageDirs=0;

   TimeBar timeBar=null;

   AudioClip timerClip=null;
   AudioClip nextClip=null;
   AudioClip gameoverClip=null;
   AudioClip clickClip=null;
   AudioClip correctClip=null;
   AudioClip incorrectClip=null;

   Image logoImage=null;

   MediaTracker tracker=null;
   Toolkit tk=null;

   LoadingBarWindow loadingBarWindow=null;
   int countImagesLoaded = 0;

   public void init() {
      gameBoard = new Panel(new GridLayout(4, 6, 4, 4));
      imageCanvas = new ImageCanvas[PANEL_NUM];

      imageDirs = Integer.parseInt(getParameter("imageDirs"));
      images = new Image[imageDirs][IMAGE_NUM];

      logoImage = getImage(getDocumentBase(), "images/logo.jpg");
      if(tracker == null) {
         tracker = new MediaTracker(this);
      }
      tracker.addImage(logoImage, 0);
      try {
         tracker.waitForAll();
      } catch(InterruptedException e) {
         System.out.println(e);
      }
      loadingBarWindow = new LoadingBarWindow((Frame)this.getParent(),
                                              logoImage, "loading images...");
      loadingBarWindow.show();

      for(int i=0;i<imageDirs;i++) {
         String dir = getParameter("imageDir["+i+"]");
         loadImages(i, dir);

         System.out.println(dir);
      }

      loadingBarWindow.setMessage("loading sounds...");
      loadingBarWindow.setStatusLevel(0);
      loadSounds();

      loadingBarWindow.dispose();

      setLayout(new BorderLayout(4, 4));
      for(int i=0;i<PANEL_NUM;i++) {
         imageCanvas[i] = new ImageCanvas();
         gameBoard.add(imageCanvas[i]);
      }

      statusPanel = new Panel();

      startStopButton = new Button("Start");
      imageLabel = new Label("Select:", Label.RIGHT);
      imageChoice = new Choice();
      powerLabel = new Label("Power:", Label.RIGHT);
      timeBar = new TimeBar(TIMEOVER);
      scoreLabel = new Label("Score:", Label.RIGHT);
      scoreField = new TextField("0", 8);
      scoreField.setEnabled(false);

      imageLabel.setForeground(Color.blue);
      powerLabel.setForeground(Color.blue);
      scoreLabel.setForeground(Color.blue);

      statusPanel.add(startStopButton);
      statusPanel.add(imageLabel);
      statusPanel.add(imageChoice);
      statusPanel.add(powerLabel);
      statusPanel.add(timeBar);
      statusPanel.add(scoreLabel);
      statusPanel.add(scoreField);

      for(int i=0;i<imageDirs;i++) {
         imageChoice.addItem(getParameter("imageDir["+i+"]"));
      }

      add("Center", gameBoard);
      add("South", statusPanel);

      Button eastBar = new Button("");
      Button westBar = new Button("");
      Button northBar = new Button("");

      add("North", northBar);
      add("West", westBar);
      add("East", eastBar);

      northBar.setEnabled(false);
      westBar.setEnabled(false);
      eastBar.setEnabled(false);

      startStopButton.addMouseListener(new MouseAdapter() {
         public void mouseEntered(MouseEvent e) {
            e.getComponent().setForeground(Color.red);
         }
         public void mouseExited(MouseEvent e) {
            e.getComponent().setForeground(Color.black);
         }
      });

      startStopButton.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            if(gameFlag == false) {
               startStopButton.setLabel("Stop");
               startGame();
               gameFlag = true;
            } else {
               startStopButton.setLabel("Start");
               stopGame();
               gameFlag = false;
            }
            clearGame();
         }
      });

      imageChoice.addItemListener(new ItemListener() {
         public void itemStateChanged(ItemEvent e) {
            if(e.getStateChange() == ItemEvent.SELECTED) {
               selectedImage = imageChoice.getSelectedIndex();

               initGame();
            }
         }
      });

      MouseListener ml2 = new MouseAdapter() {
         public void mousePressed(MouseEvent e) {
            ImageCanvas p = (ImageCanvas)e.getSource();

            clickClip.play();
            p.reverseState();
            if(oldImage != null) {
               oldImage.repaint();
            }
            if(p != null) {
               p.repaint();
            }
            try {
               Thread.sleep(MISMATCH_DELAY);
            } catch (InterruptedException ie) {
               System.out.println(e);
            }

            if(p.getState() == ImageCanvas.STATE_BACK) {
               currentFrontNum--;
               if(currentFrontNum <= 0) {
                  oldImage = null;
                  currentFrontNum = 0;
               }
            } else if(p.getState() == ImageCanvas.STATE_FRONT) {
               currentFrontNum++;
               if(currentFrontNum >= 2) {
                  if(oldImage.equals(p)) {
                     /* Correct */
                     correctClip.play();

                     matchedNum += 2;
                     oldImage.setEnabled(false);
                     p.setEnabled(false);

                     if(matchedNum >= PANEL_NUM) {
                        congraturation();
                     }
                  } else {
                     // Mismatch sound
                     incorrectClip.play();

                     oldImage.setState(ImageCanvas.STATE_BACK);
                     p.setState(ImageCanvas.STATE_BACK);

                     oldImage.repaint();
                     p.repaint();

                     currentFrontNum = 0;
                  }
                  oldImage = null;
                  currentFrontNum = 0;
               } else {
                  oldImage = p;
               }
            }
         }
      };

      for(int i=0;i<PANEL_NUM;i++) {
         imageCanvas[i].addMouseListener(ml2);
         imageCanvas[i].setEnabled(false);
      }
      initGame();
   }

   public void congraturation() {
      stopGame();

      score += (100 * level + (TIMEOVER * 1000 - elapsedTime) / 100);
      scoreField.setText(""+score);
      level++;

      WarningDialog dialog=null;
      if(level > LEVEL_MAX) {
         nextClip.play();
         dialog = new WarningDialog((Frame)this.getParent(), "Notice!",
                                    "Congraturations");
         dialog.show();
         scoreBoard();
      } else {
         nextClip.play();
         dialog = new WarningDialog((Frame)this.getParent(), "Notice!",
                                    "Level "+level);
         dialog.show();
         startGame();
      }
   }

   public void scoreBoard() {
      AppletContext appletContext = getAppletContext();
      String queryString = getCodeBase()+"ScoreBoard/ScoreBoard.asp?score="+score;
      try {
         appletContext.showDocument(new URL(queryString), "_self");
      } catch(MalformedURLException ee) {
         appletContext.showStatus("URL Exception: " + ee);
      }
   }

   public void timeover() {
      stopGame();

      gameoverClip.play();
      WarningDialog dialog = new WarningDialog((Frame)this.getParent(), "Notice!",
                                               "Game Over");
      dialog.show();

      score += (matchedNum * level * 10);
      scoreField.setText(""+score);
      scoreBoard();
   }

   public void clearGame() {
      score = 0;
      level = 0;
   }

   public void initGame() {
      int value=0;
      boolean check[] = new boolean[PANEL_NUM];

      for(int i=0;i<PANEL_NUM;i++) {
         check[i] = false;
      }

      for(int i=0;i<PANEL_NUM;) {
         value = (int)(Math.random() * PANEL_NUM * 2) % PANEL_NUM;

         if(check[value] == false) {
            check[value] = true;
            imageCanvas[value].setImage(images[selectedImage][i%IMAGE_NUM]);
            imageCanvas[value].setId(i%IMAGE_NUM);
            imageCanvas[value].setState(ImageCanvas.STATE_FRONT);

            imageCanvas[value].repaint();

            i++;
         }
      }
      oldImage = null;
      matchedNum = 0;
      currentFrontNum = 0;
   }

   public void startGame() {
      if(gameFlag == false) {
         initGame();

         gameFlag = true;
         imageChoice.setEnabled(false);

         for(int i=0;i<PANEL_NUM;i++) {
            imageCanvas[i].setState(ImageCanvas.STATE_BACK);
            imageCanvas[i].repaint();

            imageCanvas[i].setEnabled(true);
         }
         timeBar.setTime(TIMEOVER - (5*level));

         startTime = new Date().getTime();
         start();
      }
      startStopButton.setLabel("Stop");
   }

   public void stopGame() {
      stop();
      if(gameFlag == true) {
         gameFlag = false;

         imageChoice.setEnabled(true);
         for(int i=0;i<PANEL_NUM;i++) {
            imageCanvas[i].setState(ImageCanvas.STATE_FRONT);
            imageCanvas[i].repaint();
            imageCanvas[i].setEnabled(false);
         }
         timeBar.setTime(0);
      }
      startStopButton.setLabel("Start");
   }

   public void start() {
      if(gameFlag == true) {
         if(elapsedThread == null) {
            elapsedThread = new Thread(this);
         }
         elapsedThread.start();

         timerClip.loop();
      }
   }

   public void stop() {
      if(gameFlag == true) {
         timerClip.stop();
      }
      elapsedThread = null;
   }

   public void run() {
      int time;

      while(true) {
         if(gameFlag == true) {
            elapsedTime = new Date().getTime() - startTime;
            time = (int)(TIMEOVER - (5*level) - (int)(elapsedTime/1000));

            timeBar.setTime(time);

            if(time <= 0) {
               gameFlag = false;
               timeover();
               break;
            }
         }
         try {
            Thread.sleep(1000);
         } catch (InterruptedException e) { }
      }
      elapsedThread = null;
   }

   private void getImagesFromZip(int dirId, String dir) {
      try {
         URL url = new URL(getDocumentBase(), "images/"+dir+"/images.zip");

System.out.println("URL: "+url);

         DataInputStream bi = new DataInputStream(url.openStream());
         ZipInputStream zi = new ZipInputStream(bi);
         ZipEntry zip = zi.getNextEntry();
         String name = zip.getName();
         int size = (int)zip.getSize();
         byte buffer[] = null;

         if(tk == null) {
            tk = Toolkit.getDefaultToolkit();
         }
         while(zip != null) {
            name = zip.getName();
            size = (int)zip.getSize();
            buffer = new byte[size];
            int pos = 0;
            int bytesRead;
            for(;size>0;size-=bytesRead) {
               bytesRead = zi.read(buffer, pos, size);
               pos += bytesRead;
            }

            for(int i=0;i<IMAGE_NUM;i++) {
               if(("image"+(i+1)+".jpg").equals(name)) {
                  images[dirId][i] = tk.createImage(buffer);
               }
            }
            countImagesLoaded++;
            int level = (int)(((double)countImagesLoaded
                             / (double)(imageDirs*IMAGE_NUM))
                            * (double)100);
            loadingBarWindow.setStatusLevel(level);

            zip = zi.getNextEntry();
        }
      } catch(Exception anyException) {
         anyException.printStackTrace();
      }
   }

   public AudioClip getAudioClip(URL url, String name) {
      AudioClip clip = super.getAudioClip(url, name);
      clip.play();
      clip.stop();

      return(clip);
   }

   private void loadImages(int dirId, String dir) {
      if(tracker == null) {
         tracker = new MediaTracker(this);
      }
      try {
         getImagesFromZip(dirId, dir);

         for(int i=0;i<IMAGE_NUM;i++) {
            tracker.addImage(images[dirId][i], 0);
         }
      } catch(Exception ex) {
         System.out.println("Exception:"+ex);
      }
      try {
         tracker.waitForID(0);
      } catch(Exception exception1) {
      }
   }

   private void loadSounds() {
      int level=0, countSoundsLoaded = 0;

      timerClip = getAudioClip(getDocumentBase(), "audio/timer.au");
      level = (int)(((double)(++countSoundsLoaded) / (double)6) * (double)100);
      loadingBarWindow.setStatusLevel(level);

      nextClip = getAudioClip(getDocumentBase(), "audio/next.au");
      level = (int)(((double)(++countSoundsLoaded) / (double)6) * (double)100);
      loadingBarWindow.setStatusLevel(level);

      gameoverClip = getAudioClip(getDocumentBase(), "audio/gameover.au");
      level = (int)(((double)(++countSoundsLoaded) / (double)6) * (double)100);
      loadingBarWindow.setStatusLevel(level);

      clickClip = getAudioClip(getDocumentBase(), "audio/click.au");
      level = (int)(((double)(++countSoundsLoaded) / (double)6) * (double)100);
      loadingBarWindow.setStatusLevel(level);

      correctClip = getAudioClip(getDocumentBase(), "audio/correct.au");
      level = (int)(((double)(++countSoundsLoaded) / (double)6) * (double)100);
      loadingBarWindow.setStatusLevel(level);

      incorrectClip = getAudioClip(getDocumentBase(), "audio/incorrect.au");
      level = (int)(((double)(++countSoundsLoaded) / (double)6) * (double)100);
      loadingBarWindow.setStatusLevel(level);

      loadingBarWindow.setStatusLevel(100);
  }
}