package rndclub.game.snowcraft;

import java.awt.*;

public class ImageCanvas extends Canvas {
   final static int STATE_FRONT=0;
   final static int STATE_BACK=1;
   final static int INVALID_ID=-1;

   Color bgColor=null;

   Image image=null;
   int id=INVALID_ID, width, height;
   int state=STATE_BACK;

   public ImageCanvas() {
      this.id = INVALID_ID;
      this.image = null;
      init();
   }

   public ImageCanvas(int id, Image image) {
      this.id = id;
      this.image = image;
      init();
   }

   public void init() {
      state = STATE_BACK;
      if(image == null) {
         setSize(0, 0);
      }
      bgColor = new Color(0xef, 0xf8, 0xff);
   }

   public boolean equals(Object o) {
      if((o != null)&&(o instanceof ImageCanvas)) {
         ImageCanvas p = (ImageCanvas)o;
         if(p.getId() == this.id) {
            return(true);
         }
      }
      return(false);
   }

   public int getState() {
      return(state);
   }

   public int getId() {
      return(id);
   }

   public void setId(int id) {
      this.id = id;
   }

   public void reverseState() {
      if(state == STATE_FRONT) {
         state = STATE_BACK;
      } else {
         state = STATE_FRONT;
      }
   }

   public void setState(int state) {
      this.state = state;
   }

   public Image getImage() {
      return(image);
   }

   public void setImage(Image image) {
      this.image = image;
      init();
   }

   public int getWidth() {
      return(getSize().width);
   }

   public int getHeight() {
      return(getSize().height);
   }

   public void repaint() {
      paint(getGraphics());
   }

   public void update(Graphics g) {
      paint(g);
   }

   public void paint(Graphics g) {
      int width = getSize().width;
      int height = getSize().height;

      if(g == null) return;

      if((image != null)&&(state == STATE_FRONT)) {
         g.drawImage(image, 0, 0, width, height, this);
      } else {
         if(bgColor != null) {
            g.setColor(bgColor);
         } else {
            g.setColor(Color.lightGray);
         }
         g.fillRect(0, 0, width-1, height-1);
         g.setColor(Color.black);
         g.drawRect(0, 0, width-1, height-1);
      }
   }
}
